@@ -25,11 +25,14 @@
 
 static struct option long_options[] =
 {
-  {"convert", 0, 0, 'c'},
-  {"delete",  0, 0, 'd'},
-  {"list",    0, 0, 'l'},
-  {"replace", 1, 0, 'r'},
-  {"version", 0, 0, 'v'}
+  {"convert",   0, 0, 'c'},
+  {"delete",    0, 0, 'd'},
+  {"help",      0, 0, 'h'},
+  {"keepgoing", 0, 0, 'k'},
+  {"list",      0, 0, 'l'},
+  {"replace",   1, 0, 'r'},
+  {"version",   0, 0, 'v'},
+  {0, 0, 0, 0}
 };
 
 #else /* not HAVE_GETOPT_LONG */
@@ -49,6 +52,7 @@ usage(char *progname)
   printf("   -r <path>|--replace <path>  Replace current rpath/runpath setting\n");
   printf("                               with the path given\n");
   printf("   -l|--list                   List the current rpath/runpath (default)\n");
+  printf("   -h|--help                   Show this usage information.\n");
 #ifndef HAVE_GETOPT_LONG
   printf("\n *** The long options are not available on this platform");
 #endif /* not HAVE_GETOPT_LONG */
@@ -61,12 +65,14 @@ usage(char *progname)
 int
 main(int argc, char * const argv[])
 {
+  int retval = 0;
   int convert = 0;      /* convert to given type */
   int remove = 0;       /* remove or not */
+  int keep_going = 0;   /* Break on first error, or keep going? */
   char *newpath = NULL; /* insert this path */
   int opt;
 #ifdef HAVE_GETOPT_LONG
-  int option_index;
+  int option_index = 0;
 #endif /* HAVE_GETOPT_LONG */
 
   if (argc < 2)
@@ -76,7 +82,7 @@ main(int argc, char * const argv[])
     }
 
   do {
-    opt = GETOPT_LONG(argc, argv, "cdlr:v", long_options, &option_index);
+    opt = GETOPT_LONG(argc, argv, "cdhklr:v", long_options, &option_index);
     switch (opt)
       {
 #if defined(DT_RUNPATH)
@@ -87,6 +93,9 @@ main(int argc, char * const argv[])
       case 'd':
         remove = 1;
         break;
+      case 'k':
+        keep_going = 1;
+        break;
       case 'r':
         newpath = optarg;
         break;
@@ -101,19 +110,21 @@ main(int argc, char * const argv[])
         break;
       default:
         printf("Invalid argument '%c'\n", opt);
+      case 'h':
         usage(argv[0]);
         exit(0);
+        break;
       }
   } while (-1 != opt);
 
-  while (optind < argc)
+  while (optind < argc && (!retval || keep_going))
     {
       if (remove)
-        killrpath(argv[optind++]);
+        retval |= killrpath(argv[optind++]);
       else
         /* list by default, replace if path is set */
-        chrpath(argv[optind++], newpath, convert);
+        retval |= chrpath(argv[optind++], newpath, convert);
     }
 
-  return 0;
+  return retval;
 }
